#!/usr/bin/perl

eval 'exec /usr/bin/perl  -S $0 ${1+"$@"}'
    if 0; # not running under some shell

use strict;
use warnings;
use File::Spec;
use File::DXF;
use File::DXF::ENTITIES;
use File::DXF::HEADER;
use File::DXF::BLOCKS;

use lib 'lib';
use Molior;

die "usage: $0 wall.yml wall.dxf" unless scalar @ARGV == 2;

my $molior = new Molior;
$molior->Read ($ARGV[0]) || die "Can't open $ARGV[0]";

die "Error, component folder not found, please set SHARE_DIR=/path/to/share"
    unless $molior->Share_dir;

my $dxf = new File::DXF;
$dxf->{HEADER} = new File::DXF::HEADER;
$dxf->{ENTITIES} = new File::DXF::ENTITIES;
$dxf->{BLOCKS} = new File::DXF::BLOCKS;

if ($molior->Type eq 'molior-wall')
{
    my $path_db = 'share/openings.yml';
    $path_db = File::Spec->catfile ($molior->Share_dir, 'openings.yml');
    $molior->Read_Db ($path_db);

    for (0 .. $molior->Segments -1)
    {
        $molior->Fix_Segment ($_, $molior->{outer} + 0.2) || die "Can't fix segment $_";
    }

    push @{$dxf->{ENTITIES}}, $molior->Inserts;
    push @{$dxf->{ENTITIES}}, $molior->Infills;
    push @{$dxf->{ENTITIES}}, $molior->Walls;
    $dxf->{BLOCKS}->Add_From_Disk ($molior->Openings_in_use);
}
elsif ($molior->Type eq 'molior-roof')
{
    push @{$dxf->{ENTITIES}}, $molior->Roof;
}
elsif ($molior->Type eq 'molior-extrusion')
{
    push @{$dxf->{ENTITIES}}, $molior->Extrusion;
}
elsif ($molior->Type eq 'molior-ceiling')
{
    push @{$dxf->{ENTITIES}}, $molior->Ceiling;
}
elsif ($molior->Type eq 'molior-floor')
{
    push @{$dxf->{ENTITIES}}, $molior->Floor;
}
elsif ($molior->Type eq 'molior-insert')
{
    push @{$dxf->{ENTITIES}}, $molior->Insert_insert;
    $dxf->{BLOCKS}->Add_From_Disk ($molior->Insert_in_use);
}

open DXF, '>'.$ARGV[1];
binmode DXF, ":crlf";
print DXF $dxf->DXF;
close DXF;

0;

__END__

=head1 NAME

molior - Generate 3D building models

=head1 SYNOPSIS

Create a DXF 3D object from a Molior YAML definition:

  molior.pl thing.yml thing.dxf

Merge the 3DFACE entities into a single mesh:

  3dface2polyline.pl thing.dxf thing.dxf

Join two or more DXF files into a single combined DXF file:

  dxfcat.pl thing.dxf this.dxf that.dxf merged.dxf

Convert the DXF file to a Collada file:

  polyline2collada.pl merged.dxf merged.dae

=head1 DESCRIPTION

B<molior> takes a geometry definition from a YAML file and writes a DXF file
representation with 3DFACE and BLOCK references.

=head1 LICENSE

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

=head1 SEE ALSO

L<Urb> L<File::DXF> L<Molior>

=head1 AUTHOR

Bruno Postle - 2011.

=cut
